@@ -43,9 +43,9 @@ eval q[
 ###################
 
 if ($Config->{dsn} =~ /^skip$/i) {
-    plan skip_all => "User has skipped the test suite.\n"
-                   . "Run `perl Makefile.PL -s` to reconfigure the connection "
-                   . "parameters\nfor the test database.";
+    plan skip_all => "User has skipped the test suite. Run `perl Makefile.PL "
+                   . "-s` to reconfigure the connection parameters for the "
+		   . "test database.";
 }
 
 my $dbh = eval {
@@ -54,21 +54,22 @@ my $dbh = eval {
 };
 
 if (not $dbh) {
-    plan skip_all => "Couldn't connect to the database for testing.\n"
-                   . "Run `perl Makefile.PL -s` to reconfigure the connection "
-                   . "parameters\nfor the test database.";
+    plan skip_all => "Couldn't connect to the database for testing. Run `perl "
+                   . "Makefile.PL -s` to reconfigure the connection parameters "
+		   . "for the test database.";
 
 } elsif ( ! $drivers{ $dbh->{Driver}{Name} } ) {
     $dbh->disconnect;
-    plan skip_all => "Your database driver is not supported. (supported: "
-                   . join(" " => sort keys %drivers) . ")\n"
-                   . "Run `perl Makefile.PL -s` to reconfigure the connection "
-                   . "parameters\nfor the test database.";
+    my $drivers = join " " => sort keys %drivers;
+    
+    plan skip_all => "Your database driver is not supported (supported: "
+                   . "$drivers). Run `perl Makefile.PL -s` to reconfigure "
+		   . "the connection parameters for the test database.";
 
 } else {
-    plan tests => 67;
-    print "# Starting test suite. Run `perl Makefile.PL -s` to reconfigure\n"
-            . "# connection parameters for the test database.\n";
+    plan tests => 70;
+    diag "Starting test suite. Run `perl Makefile.PL -s` to reconfigure "
+       . "connection parameters for the test database.";
 }
 
 ## clear all tables first
@@ -78,7 +79,7 @@ my $driver = $dbh->{Driver}{Name};
 my $q = $dbh->prepare( $drivers{$driver}{show} );
 $q->execute;
 while ( my ($table) = $q->fetchrow_array ) {
-    print "DROPPING $table\n";
+    diag "DROPPING $table";
     $dbh->do("drop table $table");
 }
 $q->finish;
@@ -114,7 +115,7 @@ $dbh->do($_) for (split /\s*;\s*/, <<"END_OF_SQL");
     create table products (
         id              $pkey,
         name            varchar(50) not null,
-        weight          integer not null,
+        weight          integer,
         price           decimal
     );
     
@@ -323,9 +324,20 @@ my %simple_accessors = (
     }
 );
 
+my $field_ok = 1;
+
 for my $class (keys %simple_accessors) {
     my $obj = $class->fetch(1);
 
+    my @supposed_accessors = $obj->field;
+
+    @supposed_accessors == @{[ keys %{ $simple_accessors{$class} } ]}
+        or $field_ok = 0;
+
+    for (@supposed_accessors) {
+        exists $simple_accessors{$class}{$_} or $field_ok = 0;
+    }
+
     for my $accessor (keys %{ $simple_accessors{$class} }) {
         is( ref $obj->$accessor,
             $simple_accessors{$class}{$accessor},
@@ -333,6 +345,9 @@ for my $class (keys %simple_accessors) {
     }
 }
 
+ok( $field_ok,
+    "field() accessor with no args" );
+
 ####
 
 my $h = Employees->fetch(1);
@@ -507,6 +522,30 @@ is( scalar Employees->search,
     undef,
     "delete all in a table" );
 
+#######################
+## undef/null values ##
+#######################
+
+{
+    my $prod = Products->fetch(4);
+    $prod->weight(undef);
+
+    my $ar = $dbh->selectall_arrayref("select weight from products where id=4");
+    
+    ok( exists $ar->[0][0] && ! defined $ar->[0][0],
+        "undef becomes NULL in database" );
+
+}
+
+## flyweight object goes out of scope, so next line must query the database
+
+ok( ! defined Products->fetch(4)->weight,
+    "NULL database values come back undef" );
+
+is( scalar( () = Products->search( weight => undef ) ),
+    1,
+    "search method handles searching with IS NULL" );
+
 ###########################
 ## subclassing revisited ##
 ###########################
@@ -519,8 +558,7 @@ isnt(
 ## done!
 
 $timer = times - $timer;
-ok( 1,
-    "summary: $Class::Tables::SQL_QUERIES queries, ${timer}s ($driver)" );
+diag "SUMMARY: $Class::Tables::SQL_QUERIES queries, ${timer}s (using $driver)";
 
 #############
 ## cleanup ##
